using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Text.RegularExpressions;

namespace DragonChat
{
    public class Connection
    {
        public delegate void ConnectionHandler();
        public delegate void DataArrivalHandler(string data);

        public event ConnectionHandler OnConnect;
        public event ConnectionHandler OnDisconnect;
        public event DataArrivalHandler OnDataArrival;

        private Socket client = null;
        private byte[] buffer = new byte[1024];
        private int size = 1024;

        private bool m_IsConnected = false;
        private int m_Port = 6667;
        private string m_Server = "irc.buzzen.net";

        public Connection()
        {
        }

        ~Connection()
        {
            if (m_IsConnected == true)
                Close();
        }
        /// <summary>
        /// Gets value indicating wether a connection is open. 
        /// </summary>
        public bool IsConnected
        {
            get
            {
                return m_IsConnected;

            }
        }
        /// <summary>
        /// Gets value containing the current server address.
        /// </summary>
        public string ServerAddress
        {
            get
            {
                return m_Server;
            }
        }
        /// <summary>
        /// Gets value containing the current port number. 
        /// </summary>
        public int Port
        {
            get
            {
                return m_Port;
            }
        }

        void Connected(IAsyncResult ar)
        {
            m_IsConnected = true;
            if (OnConnect != null)
                OnConnect();
            try
            {
                client.BeginReceive(buffer, 0, size, SocketFlags.None, new AsyncCallback(DataArrival), client);
            }
            catch (SocketException ex)
            {
                Close();
                System.Windows.Forms.MessageBox.Show(ex.Message, "Connected");
            }
        }

        void DataArrival(IAsyncResult ar)
        {
            try
            {
                Socket remote = (Socket)ar.AsyncState;
                int recv = remote.EndReceive(ar);
                if (recv == 0)
                {
                    Close();
                    return;
                }
                if (OnDataArrival != null) OnDataArrival(Encoding.Default.GetString(buffer, 0, recv));

                remote.BeginReceive(buffer, 0, size, SocketFlags.None, new AsyncCallback(DataArrival), remote);
            }
            catch (ObjectDisposedException)
            {
                // do nothing
            }
            catch (SocketException)
            {
                Close();
            }
        }
        /// <summary>
        /// Connects to a server using the current ServerAddress and Port values.
        /// </summary>
        public void Connect()
        {
            Connect(m_Server, m_Port);
        }
        /// <summary>
        /// Connects to a server using the address specified and the current Port value.
        /// </summary>
        public void Connect(string server)
        {
            Connect(server, m_Port);
        }
        /// <summary>
        /// Connects to a server using the address and port specified.
        /// </summary>
        public void Connect(string server, int port)
        {
            if (!m_IsConnected)
            {
                m_Server = server;
                m_Port = port;
                try
                {
                    this.client = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                    IPHostEntry ip = Dns.GetHostEntry(server);
                    this.client.BeginConnect(new IPEndPoint(ip.AddressList[0], m_Port), new AsyncCallback(Connected), client);
                }
                catch (SocketException se)
                {
                    Close();
                    throw se;
                }
                catch (Exception ex)
                {
                    Close();
                    System.Windows.Forms.MessageBox.Show(ex.Message, "Connect");
                }
            }
            else
            {
                Close();
                Connect(server, port);
            }
        }
        /// <summary>
        /// Sends data to current server if a connection is available. 
        /// </summary>
        public void SendData(string text)
        {
            try
            {
                byte[] data = Encoding.UTF8.GetBytes(text);
                client.Send(data, 0, data.Length, SocketFlags.None);
            }
            catch (SocketException)
            {
                Close();
            }
        }
        /// <summary>
        /// Closes connection to server if a connection is open. 
        /// </summary>
        public void Close()
        {
            if (m_IsConnected)
            {
                try
                {
                    if (OnDisconnect != null) OnDisconnect();
                    client.Shutdown(SocketShutdown.Both);
                    client.Close();
                    m_IsConnected = false;
                }
                catch (SocketException)
                {
                    m_IsConnected = false;
                }
                catch (ObjectDisposedException)
                {
                    // do nothing
                }
            }
        }
    }
}

