using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Runtime.InteropServices;

namespace DragonChat
{
    public partial class WhisperForm : Form
    {
        private string _nickname = "";

        private bool focused = false;

        delegate void CmdCallback();
        delegate void SetTextCallback(string text);
        delegate void DisplayCallback(string nick, string text);
        delegate void SetColorTextCallback(string text, System.Windows.Media.Color color, bool bold, bool italic);

        public delegate void WhisperInputCallback(WhisperForm whisper, string text);
        public delegate void WhisperCloseCallback(WhisperForm whisper);

        public event WhisperInputCallback OnWhisperInput;
        public event WhisperCloseCallback OnWhisperClose;

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool FlashWindowEx(ref FLASHWINFO pwfi);

        [StructLayout(LayoutKind.Sequential)]
        public struct FLASHWINFO
        {
            public UInt32 cbSize;
            public IntPtr hwnd;
            public UInt32 dwFlags;
            public UInt32 uCount;
            public UInt32 dwTimeout;
        }

        //Stop flashing. The system restores the window to its original state. 
        public const UInt32 FLASHW_STOP = 0;
        //Flash the window caption. 
        public const UInt32 FLASHW_CAPTION = 1;
        //Flash the taskbar button. 
        public const UInt32 FLASHW_TRAY = 2;
        //Flash both the window caption and taskbar button.
        //This is equivalent to setting the FLASHW_CAPTION | FLASHW_TRAY flags. 
        public const UInt32 FLASHW_ALL = 3;
        //Flash continuously, until the FLASHW_STOP flag is set. 
        public const UInt32 FLASHW_TIMER = 4;
        //Flash continuously until the window comes to the foreground. 
        public const UInt32 FLASHW_TIMERNOFG = 12;

        private const int WS_CLIPCHILDREN = 0x02000000;
        private const int WS_CLIPSIBLINGS = 0x04000000;

        public WhisperForm(string nickname)
        {
            InitializeComponent();
            this.Nickname = nickname;
            
            this.FormClosed += new FormClosedEventHandler(WhisperForm_FormClosed);
            this.Deactivate += new EventHandler(WhisperForm_Deactivate);
            this.Activated += new EventHandler(WhisperForm_Activated);

            this.rtbInput.TextChanged += new EventHandler(rtbInput_TextChanged);
            this.rtbInput.KeyDown += new KeyEventHandler(rtbInput_KeyDown);

            this.rtbOutput.MouseUp += new System.Windows.Input.MouseButtonEventHandler(rtbOutput_MouseUp);
            this.rtbOutput.MouseDoubleClick += new System.Windows.Input.MouseButtonEventHandler(rtbOutput_MouseDoubleClick);
            this.rtbOutput.TextChanged += new TextChangedEventHandler(rtbOutput_TextChanged);
        }

        protected override CreateParams CreateParams
        {
            get
            {
                CreateParams cp = base.CreateParams;
                cp.Style |= WS_CLIPCHILDREN;
                cp.Style |= WS_CLIPSIBLINGS;
                return cp;
            }
        }

        void rtbOutput_TextChanged(object sender, TextChangedEventArgs e)
        {
        }

        void rtbOutput_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            rtbOutput.Selection.Select(rtbOutput.CaretPosition, rtbOutput.CaretPosition);
            e.Handled = true;
        }

        void rtbOutput_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            //throw new NotImplementedException();
        }

        void WhisperForm_Activated(object sender, EventArgs e)
        {
            focused = true;
            StopFlash(this);
        }

        void WhisperForm_Deactivate(object sender, EventArgs e)
        {
            focused = false;
        }

        public string Nickname
        {
            get
            {
                return _nickname;
            }
            set
            {
                _nickname = value;
                SetTitle(value);
            }
        }

        private void WhisperForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (OnWhisperClose != null)
                OnWhisperClose(this);
        }

        private void rtbInput_TextChanged(object sender, EventArgs e)
        {
            if (rtbInput.Text.IndexOf('\n') > -1)
            {
                string[] line = rtbInput.Text.Replace("\r", "").Split(new string[] { "\n" }, StringSplitOptions.RemoveEmptyEntries);
                for (int i = 0; i < line.Length; i++)
                    if (OnWhisperInput != null) OnWhisperInput(this, line[i]);
                rtbInput.Clear();
            }
        }

        private void rtbInput_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control)
                if (e.KeyCode == Keys.V)
                {
                    rtbInput.Paste(DataFormats.GetFormat(DataFormats.Text));
                    e.Handled = true;
                }
            if (e.Shift)
                if (e.KeyCode == Keys.Insert)
                {
                    rtbInput.Paste(DataFormats.GetFormat(DataFormats.Text));
                    e.Handled = true;
                }
            if (e.KeyCode == Keys.Enter)
            {
                string[] line = rtbInput.Text.Replace("\r", "").Split(new string[] { "\n" }, StringSplitOptions.RemoveEmptyEntries);
                for (int i = 0; i < line.Length; i++)
                    if (OnWhisperInput != null) OnWhisperInput(this, line[i]);
                rtbInput.Clear();
                e.Handled = true;
            }
        }

        private void SetTitle(string text)
        {
            if (this.InvokeRequired)
                this.Invoke(new SetTextCallback(this.SetTitle), new object[] { text });
            else
                this.Text = "Whisper: " + text;
        }

        public void Clear()
        {
            if (rtbOutput.Dispatcher.CheckAccess())
            {
                rtbOutput.Document.Blocks.Clear();
            }
            else
            {
                rtbOutput.Dispatcher.Invoke(new CmdCallback(this.Clear));
            }
        }

        public void DisplayWhisper(string nick, string text)
        {
            if (rtbOutput.Dispatcher.CheckAccess())
            {
                StringBuilder sb = new StringBuilder();
                if ((new TextRange(rtbOutput.Document.ContentStart, rtbOutput.Document.ContentEnd)).Text.Length > 2)
                {
                    new LineBreak(rtbOutput.Document.ContentEnd);
                }
                if (nick == _nickname)
                    sb.Append(MakeStyle(rtbOutput.FontFamily.Source, Colors.Navy, false, false));
                else
                    sb.Append(MakeStyle(rtbOutput.FontFamily.Source, Colors.Black, true, false));
                sb.Append(nick);
                sb.Append("[/style]");
                sb.Append(" : ");
                sb.Append(text);
                rtbOutput.Append(sb.ToString());
                if (!rtbOutput.IsFocused)
                    rtbOutput.ScrollToEnd();
                if (!focused)
                    FlashWindowEx(this);
            }
            else
            {
                rtbOutput.Dispatcher.Invoke(new DisplayCallback(this.DisplayWhisper), new object[] { nick, text });
            }
        }

        public void SetText(string text)
        {
            if (rtbOutput.Dispatcher.CheckAccess())
            {
                if ((new TextRange(rtbOutput.Document.ContentStart,rtbOutput.Document.ContentEnd)).Text.Length  > 2)
                {
                    new LineBreak(rtbOutput.Document.ContentEnd);
                }
                rtbOutput.Append(text);
                if (!rtbOutput.IsFocused)
                    rtbOutput.ScrollToEnd();
            }
            else
            {
                rtbOutput.Dispatcher.Invoke(new SetTextCallback(this.SetText), new object[] { text });
            }
        }

        public void SetText(string text, System.Windows.Media.Color color, bool bold, bool italic)
        {
            if (rtbOutput.Dispatcher.CheckAccess())
            {
                StringBuilder sb = new StringBuilder();
                if ((new TextRange(rtbOutput.Document.ContentStart, rtbOutput.Document.ContentEnd)).Text.Length > 2)
                {
                    new LineBreak(rtbOutput.Document.ContentEnd);
                }
                sb.Append(MakeStyle(rtbOutput.FontFamily.Source, color, bold, italic));
                sb.Append(text);
                sb.Append("[/style]");
                rtbOutput.Append(sb.ToString());
                if (!rtbOutput.IsFocused)
                    rtbOutput.ScrollToEnd();
            }
            else
            {
                rtbOutput.Dispatcher.Invoke(new SetColorTextCallback(this.SetText), new object[] { text, color, bold, italic });
            }
        }

        private static void FlashWindowEx(WhisperForm frm)
        {
            IntPtr hWnd = frm.Handle;
            FLASHWINFO fInfo = new FLASHWINFO();

            fInfo.cbSize = Convert.ToUInt32(Marshal.SizeOf(fInfo));
            fInfo.hwnd = hWnd;
            fInfo.dwFlags = FLASHW_ALL;
            fInfo.uCount = UInt32.MaxValue;
            fInfo.dwTimeout = 0;


            bool flash = FlashWindowEx(ref fInfo);
        }

        private static void StopFlash(WhisperForm frm)
        {
            IntPtr hWnd = frm.Handle;
            FLASHWINFO fInfo = new FLASHWINFO();

            fInfo.cbSize = Convert.ToUInt32(Marshal.SizeOf(fInfo));
            fInfo.hwnd = hWnd;
            fInfo.dwFlags = FLASHW_STOP;

            bool flash = FlashWindowEx(ref fInfo);
        }

        private string MakeStyle(string fontname, System.Windows.Media.Color fontcolor, bool bold, bool italic)
        {
            StringBuilder sb = new StringBuilder("[style ff:");
            sb.Append(fontname);
            sb.Append(";co:");
            sb.Append(string.Format("#{0}", fontcolor.ToString().Substring(3)));
            if (bold)
                sb.Append(";b");
            if (italic)
                sb.Append(";i");
            sb.Append(";]");
            return sb.ToString();
        }
    }
}